==> lhcsmapi-1.2.130.tar.gz: lhcsmapi-1.2.130/lhcsmapi/analysis/RQCircuitQuery.py <==
import warnings
import numpy as np
import pandas as pd

import lhcsmapi
from lhcsmapi.Time import Time
from lhcsmapi.analysis.CircuitQuery import CircuitQuery
from lhcsmapi.metadata.MappingMetadata import MappingMetadata
from lhcsmapi.pyedsl.QueryBuilder import QueryBuilder
from lhcsmapi.reference.Reference import Reference


class RQCircuitQuery(CircuitQuery):

    def __init__(self, circuit_type, circuit_name):
        super().__init__(circuit_type, circuit_name)

    def find_timestamp_pic(self, timestamp_fgc, *, spark, duration=[(1, 's'), (60, 's')], verbose=True):
        if verbose:
            print('Finding NXCALS PIC timestamp of {} at {} for {}.'.format(self.circuit_name, timestamp_fgc, duration))

        st_abort_pic_df = QueryBuilder().with_db('NXCALS').with_circuit_type(self.circuit_type)\
            .with_duration(t_start=int(timestamp_fgc), duration=duration) \
            .with_metadata(circuit_name=self.circuit_name, system='PIC', signal='ST_ABORT_PIC')\
            .signal_query(spark).dfs[0]
        return st_abort_pic_df.index[0]

    def create_report_analysis_template(self, source_timestamp_qds_df, timestamp_fgc_rqd,
                                        i_meas_rqd_df, i_meas_rqf_df, author):
        COLUMN_NAMES = ['Circuit', 'Date', 'Timestamp iQPS', 'iQPS/nQPS', 'I_q_D [A]', 'I_q_F [A]',
                        'Position', 'QPS Crate', 'Origin', 'du/dt [V/s]', 'QH', 'R_LEADS_RQD [uOhm]',
                        'R_LEADS_RQF [uOhm]', "comments", "doneBy", "version"]
        report_qds = pd.DataFrame(columns=COLUMN_NAMES)

        for index, row in source_timestamp_qds_df.iterrows():
            timestamp_qds = row['timestamp']

            t_quench = (timestamp_qds - timestamp_fgc_rqd) * 1e-9
            if t_quench > i_meas_rqd_df.index[-1]:
                i_quench_rqd = i_meas_rqd_df.values[-1]
                i_quench_rqf = i_meas_rqf_df.values[-1]
                warnings.warn('Quench time beyond PM buffer. Need to query NXCALS in order to get the quench current!')
            else:
                i_quench_rqd = int(i_meas_rqd_df.reindex([t_quench], method='nearest', tolerance=.1).values[0])
                i_quench_rqf = int(i_meas_rqf_df.reindex([t_quench], method='nearest', tolerance=.1).values[0])

            qps_crate = MappingMetadata.get_crate_name_from_magnet_name(self.circuit_type, row['source'])
            values = [self.circuit_name.replace('D', ''), Time.to_string_short(timestamp_fgc_rqd),
                      Time.to_string_short(timestamp_qds), '', i_quench_rqd, i_quench_rqf, row['source'],
                      qps_crate, '', np.nan, '', '', '', '', author, lhcsmapi.__version__]
            report_qds.loc[len(report_qds.index)] = values

        return report_qds

    def query_voltage_qh_pm(self, source_timestamp_qds_df, verbose=True):
        u_hds_dfs = []
        for index, row in source_timestamp_qds_df.iterrows():
            timestamp_qds = row['timestamp'] + 1000000
            source_qds = row['source']

            u_hds_df = QueryBuilder().with_db('PM').with_circuit_type(self.circuit_type).with_timestamp(timestamp_qds) \
                .with_metadata(circuit_name=self.circuit_name, system='QH', signal='U_HDS', source=source_qds) \
                .signal_query().synchronize_time(timestamp_qds).convert_index_to_sec().dfs

            u_hds_dfs.append(u_hds_df)

        return u_hds_dfs

    def query_voltage_qh_ref_pm(self, source_timestamp_qds_df, verbose=True):
        u_hds_ref_dfs = []
        for index, row in source_timestamp_qds_df.iterrows():
            source_qds = row['source']

            timestamp_qh_ref = Reference.get_quench_heater_reference_discharge(self.circuit_type, source_qds)
            u_hds_ref_df = QueryBuilder().with_db('PM').with_circuit_type(self.circuit_type) \
                .with_timestamp(timestamp_qh_ref) \
                .with_metadata(circuit_name=self.circuit_name, system='QH', signal='U_HDS', source=source_qds) \
                .signal_query().synchronize_time(timestamp_qh_ref).convert_index_to_sec().dfs

            u_hds_ref_dfs.append(u_hds_ref_df)

        return u_hds_ref_dfs

    def find_timestamp_leads(self, timestamp_fgc_rq, duration=[(10, 's'), (400, 's')],
                             verbose=True):
        if verbose:
            print('Finding PM DFB timestamps of {} at {} for {}.'.format(self.circuit_name, timestamp_fgc_rq, duration))

        return QueryBuilder().with_db('PM').with_circuit_type(self.circuit_type)\
            .with_duration(t_start=int(timestamp_fgc_rq), duration=duration) \
            .with_metadata(circuit_name=self.circuit_name, system='LEADS').event_query().df

    def query_leads_res_hts_voltage(self, timestamp_fgc_rq, source_timestamp_leads_rq_df, *,
                                    signal_names, spark, duration=[(30, 's'), (90, 's')], verbose=True):

        if source_timestamp_leads_rq_df.empty:
            if verbose:
                print('Querying NXCALS DFB at {} for {} of {}, {}.'.format(timestamp_fgc_rq, duration,
                                                                         self.circuit_name, signal_names))

            return QueryBuilder().with_db('NXCALS').with_circuit_type(self.circuit_type) \
                .with_duration(t_start=int(timestamp_fgc_rq), duration=duration) \
                .with_metadata(circuit_name=self.circuit_name, system='LEADS', signal=signal_names) \
                .signal_query(spark).synchronize_time(timestamp_fgc_rq).convert_index_to_sec().dfs
        else:
            if verbose:
                print('Querying PM at {} for DFB signals of {}, {}.'.format(timestamp_fgc_rq, self.circuit_name,
                                                                            signal_names))

            source_leads = source_timestamp_leads_rq_df.loc[0, 'source']
            timestamp_leads = source_timestamp_leads_rq_df.loc[0, 'timestamp']

            return QueryBuilder().with_db('PM').with_circuit_type(self.circuit_type).with_timestamp(timestamp_leads) \
                .with_metadata(circuit_name=self.circuit_name, system='LEADS', signal='U_HTS',
                               source=source_leads)\
                .signal_query(spark).synchronize_time(timestamp_leads).convert_index_to_sec().dfs

==> lhcsmapi-1.3.41.tar.gz: lhcsmapi-1.3.41/lhcsmapi/analysis/RQCircuitQuery.py <==
import warnings
import numpy as np
import pandas as pd

import lhcsmapi
from lhcsmapi.Time import Time
from lhcsmapi.analysis.CircuitQuery import CircuitQuery
from lhcsmapi.metadata.MappingMetadata import MappingMetadata
from lhcsmapi.pyedsl.QueryBuilder import QueryBuilder
from lhcsmapi.reference.Reference import Reference


class RQCircuitQuery(CircuitQuery):

    def __init__(self, circuit_type, circuit_name):
        super().__init__(circuit_type, circuit_name)

    def find_timestamp_pic(self, timestamp_fgc, *, spark, duration=[(1, 's'), (60, 's')], verbose=True):
        if verbose:
            print('Finding NXCALS PIC timestamp of {} at {} for {}.'.format(self.circuit_name, timestamp_fgc, duration))

        st_abort_pic_df = QueryBuilder().with_nxcals(spark) \
            .with_duration(t_start=int(timestamp_fgc), duration=duration) \
            .with_circuit_type(self.circuit_type) \
            .with_metadata(circuit_name=self.circuit_name, system='PIC', signal='ST_ABORT_PIC') \
            .signal_query().dfs[0]
        return st_abort_pic_df.index[0]

    def create_report_analysis_template(self, source_timestamp_qds_df, timestamp_fgc_rqd,
                                        i_meas_rqd_df, i_meas_rqf_df, author):
        COLUMN_NAMES = ['Circuit', 'Date', 'Timestamp iQPS', 'iQPS/nQPS', 'I_q_D [A]', 'I_q_F [A]',
                        'Position', 'QPS Crate', 'Origin', 'du/dt [V/s]', 'QH', 'R_LEADS_RQD [uOhm]',
                        'R_LEADS_RQF [uOhm]', "comments", "doneBy", "version"]
        report_qds = pd.DataFrame(columns=COLUMN_NAMES)

        for index, row in source_timestamp_qds_df.iterrows():
            timestamp_qds = row['timestamp']

            t_quench = (timestamp_qds - timestamp_fgc_rqd) * 1e-9
            if t_quench > i_meas_rqd_df.index[-1]:
                i_quench_rqd = i_meas_rqd_df.values[-1]
                i_quench_rqf = i_meas_rqf_df.values[-1]
                warnings.warn('Quench time beyond PM buffer. Need to query NXCALS in order to get the quench current!')
            else:
                i_quench_rqd = int(i_meas_rqd_df.reindex([t_quench], method='nearest', tolerance=.1).values[0])
                i_quench_rqf = int(i_meas_rqf_df.reindex([t_quench], method='nearest', tolerance=.1).values[0])

            qps_crate = MappingMetadata.get_crate_name_from_magnet_name(self.circuit_type, row['source'])
            values = [self.circuit_name.replace('D', ''), Time.to_string_short(timestamp_fgc_rqd),
                      Time.to_string_short(timestamp_qds), '', i_quench_rqd, i_quench_rqf, row['source'],
                      qps_crate, '', np.nan, '', '', '', '', author, lhcsmapi.__version__]
            report_qds.loc[len(report_qds.index)] = values

        return report_qds

    def query_voltage_qh_pm(self, source_timestamp_qds_df, verbose=True):
        u_hds_dfs = []
        for index, row in source_timestamp_qds_df.iterrows():
            timestamp_qds = row['timestamp'] + 1000000
            source_qds = row['source']

            u_hds_df = QueryBuilder().with_pm() \
                .with_timestamp(timestamp_qds) \
                .with_circuit_type(self.circuit_type) \
                .with_metadata(circuit_name=self.circuit_name, system='QH', signal='U_HDS', source=source_qds,
                               wildcard={'CELL': source_qds}) \
                .signal_query() \
                .synchronize_time(timestamp_qds) \
                .convert_index_to_sec().dfs

            u_hds_dfs.append(u_hds_df)

        return u_hds_dfs

    def query_voltage_qh_ref_pm(self, source_timestamp_qds_df, verbose=True):
        u_hds_ref_dfs = []
        for index, row in source_timestamp_qds_df.iterrows():
            source_qds = row['source']

            timestamp_qh_ref = Reference.get_quench_heater_reference_discharge(self.circuit_type, source_qds)
            u_hds_ref_df = QueryBuilder().with_pm() \
                .with_timestamp(timestamp_qh_ref) \
                .with_circuit_type(self.circuit_type) \
                .with_metadata(circuit_name=self.circuit_name, system='QH', signal='U_HDS', source=source_qds,
                               wildcard={'CELL': source_qds}) \
                .signal_query() \
                .synchronize_time(timestamp_qh_ref) \
                .convert_index_to_sec().dfs

            u_hds_ref_dfs.append(u_hds_ref_df)

        return u_hds_ref_dfs

    def find_timestamp_leads(self, timestamp_fgc_rq, duration=[(10, 's'), (400, 's')],
                             verbose=True):
        if verbose:
            print('Finding PM DFB timestamps of {} at {} for {}.'.format(self.circuit_name, timestamp_fgc_rq, duration))

        return QueryBuilder().with_pm() \
            .with_duration(t_start=int(timestamp_fgc_rq), duration=duration) \
            .with_circuit_type(self.circuit_type) \
            .with_metadata(circuit_name=self.circuit_name, system='LEADS') \
            .event_query().df

    def query_leads_res_hts_voltage(self, timestamp_fgc_rq, source_timestamp_leads_rq_df, *,
                                    signal_names, spark, duration=[(30, 's'), (90, 's')], verbose=True):

        if source_timestamp_leads_rq_df.empty:
            if verbose:
                print('Querying NXCALS DFB at {} for {} of {}, {}.'.format(timestamp_fgc_rq, duration,
                                                                         self.circuit_name, signal_names))

            return QueryBuilder().with_nxcals(spark) \
                .with_duration(t_start=int(timestamp_fgc_rq), duration=duration) \
                .with_circuit_type(self.circuit_type) \
                .with_metadata(circuit_name=self.circuit_name, system='LEADS', signal=signal_names) \
                .signal_query() \
                .synchronize_time(timestamp_fgc_rq) \
                .convert_index_to_sec().dfs
        else:
            if verbose:
                print('Querying PM at {} for DFB signals of {}, {}.'.format(timestamp_fgc_rq, self.circuit_name,
                                                                            signal_names))

            source_leads = source_timestamp_leads_rq_df.loc[0, 'source']
            timestamp_leads = source_timestamp_leads_rq_df.loc[0, 'timestamp']

            return QueryBuilder().with_pm() \
                .with_timestamp(timestamp_leads) \
                .with_circuit_type(self.circuit_type) \
                .with_metadata(circuit_name=self.circuit_name, system='LEADS', signal='U_HTS', source=source_leads) \
                .signal_query(spark) \
                .synchronize_time(timestamp_leads) \
                .convert_index_to_sec().dfs
